
/*
 *          |          _                     _______________________
 *          |\       _/ \_                  |                       |
 *          | \_    /_    \_                |    Alexandre Botao    |
 *          \   \__/  \__   \               |     www.botao.org     |
 *           \_    \__/  \_  \              |    55-11-8244-UNIX    |
 *             \_   _/     \ |              |  alexandre@botao.org  |
 *               \_/        \|              |_______________________|
 *                           |
 */

/*	 _______________________________________________________________
 *	|								|
 *	|	xdf				Exquisite `df'(1)	|
 *	|_______________________________________________________________|
 */

# define SWNAME	"xdf"
# define SWVERS	"1.4.34"
# define SWFORG	"$"				/*	"$" = stable	*/
# define SWDATE	"2010-12-15"
# define SWCOPY	"(fos)"
# define SWAUTH	"alexandre@botao.org"

/*____________________________________________________________________________
*/
# include "config.h"

# include <time.h>
# include <stdio.h>
# include <string.h>
# include <signal.h>
# include <unistd.h>
# include <sys/utsname.h>

# include "abcs.h"

/*	__	__	__	__	__	__	__	__	*/

# define	USE_STATVFS	/* preferably ...	*/

# ifdef CYGWIN
# define	ulong_t		unsigned long
# endif /* CYGWIN */

# ifdef LINUX
# include <stdint.h>
# define ulong_t	ulong
# endif /* LINUX */

# ifdef USE_STATFS
# include <sys/vfs.h>
struct statfs	sfsbuf ;
int32_t	x_bavail;  /* free blocks available to non-superuser */
int32_t	x_bfree;   /* free blocks */
int32_t	x_blocks;  /* total blocks in file system */
int32_t	x_bsize;   /* fundamental file system block size in bytes */
int32_t x_bused;
# endif /* USE_STATFS */

# ifdef USE_STATVFS
# include <sys/types.h>
# include <sys/statvfs.h>
struct statvfs	svfsbuf ;
ulong_t x_bsize;              /* preferred file system block size */
ulong_t x_frsize;             /* fundamental file system block size */
ulong_t x_blocks;             /* total blocks of f_frsize on file system */
ulong_t x_size;               /* size of file system in f_frsize unit */
ulong_t x_bfree;              /* free blocks */
ulong_t x_bavail;             /* blocks available to non-superuser */
ulong_t x_bused;
# endif /* USE_STATVFS */

# ifdef AIX
# include <mntent.h>
# define X_FILESYSLIST	MNTTAB		/* "/etc/filesystems"	*/
# define X_MOUNTEDLIST	MOUNTED		/* "/etc/mtab"		*/
# define XDF_IGNLST "procfs"
# define XDF_SYSLST "/","/home","/opt","/proc","/usr","/var","/tmp","/var/adm/ras/livedump"
# endif /* AIX */

# ifdef HPUX
# include <mntent.h>
# define X_FILESYSLIST	MNT_CHECKLIST	/* "/etc/fstab"		*/
#   ifdef HPUX1111
#	define X_MOUNTEDLIST	"/etc/fstab"	/* avoid hangs */
#   else
#	define X_MOUNTEDLIST	MNT_MNTTAB	/* "/etc/mnttab"	*/
#   endif
# define XDF_IGNLST "swap"
# define XDF_SYSLST "/","/home","/opt","/stand","/usr","/var","/tmp"
# endif /* HPUX */

# ifdef LINUX
# include <mntent.h>
# define X_FILESYSLIST	MNTTAB		/* "/etc/fstab"		*/
# define X_MOUNTEDLIST	MOUNTED		/* "/etc/mtab"		*/
# define XDF_IGNLST "proc","tmpfs","sysfs","devpts","fusectl","nfsd","securityfs","binfmt_misc","fuse.gvfs-fuse-daemon"
# define XDF_SYSLST "/","/boot","/dev","/home","/opt","/usr","/var","/tmp"
# endif /* LINUX */

# ifdef CYGWIN
# include <mntent.h>
# define X_FILESYSLIST	MNTTAB		/* "/etc/fstab"		*/
# define X_MOUNTEDLIST	MOUNTED		/* "/etc/mtab"		*/
# define XDF_IGNLST "proc","C:/cygwin","C:/cygwin/bin","C:/cygwin/lib"
# define XDF_SYSLST "/","/usr/bin","/usr/lib"
# endif /* CYGWIN */

# ifdef SUNX
# include <sys/mnttab.h>
# define X_FILESYSLIST	"/etc/vfstab"
# define X_MOUNTEDLIST	MNTTAB		/* "/etc/mnttab"	*/
# define setmntent(N,M)	fopen(N,M)
# define endmntent(P)	fclose(P)
# define XDF_IGNLST "ctfs","proc","mnttab","tmpfs","objfs","sharefs","fd"
# define XDF_SYSLST "/","/devices","/opt","/usr","/var","/tmp","/etc/svc/volatile","/var/run"
# endif /* SUNX */

# define	XDF_STR		0x0001
# define	XDF_INT		0x0002
# define	XDF_BIG		0x0004
# define	XDF_PCT		0x0008
# define	XDF_UNT		0x0020
# define	XDF_SOL		0x0040
# define	XDF_EOL		0x0080
# define	XDF_HEAD	0x0100
# define	XDF_PLUS	0x0200
# define	XDF_TAIL	0x0400
# define	XDF_FOOT	XDF_TAIL
# define	XDF_ITEM	0x0800
# define	XDF_BODY	XDF_ITEM

# define	MAXDIRLST	1024
# define	DFLTIMOUT	2

long long	xfs_blksiz ;
long long	xfs_totbyt ;
long long	xfs_usebyt ;
long long	xfs_avabyt ;

int		xfs_caperc ;
int		xfs_tmperc ;	/* temporary '%free' ( 100 - caperc ) */

int		sumcaperc = 0LL ;
int		sumtmperc = 0LL ;

long long	sumtotbyt = 0LL ;
long long	sumusebyt = 0LL ;
long long	sumavabyt = 0LL ;

long long	unit = 1 ;

int	borderflag = 0 ;
int	frameflag = 0 ;
int	identflag = 0 ;
int	fstabflag = 0 ;
int	verbflag = 0 ;
int	echoflag = 0 ;
int	hostflag = 0 ;
int	ansiflag = 0 ;
int	kiloflag = 0 ;
int	megaflag = 0 ;
int	gigaflag = 0 ;
int	teraflag = 0 ;
int	htmlflag = 0 ;
int	helpflag = 0 ;
int	listflag = 0 ;
int	freeflag = 0 ;
int	sumflag = 0 ;
int	volflag = 0 ;
int	sizflag = 0 ;
int	useflag = 0 ;
int	avaflag = 0 ;
int	capflag = 0 ;
int	legflag = 0 ;
int	csvflag = 0 ;
int	hdrflag = 0 ;
int	sysflag = 0 ;
int	regflag = 0 ;
int	nfsflag = 0 ;

int	volcount = 0 ;
int	timedout = 0 ;
int	xtimeout = DFLTIMOUT ;

int	hostwid = 12 ;
int	volwid = 24 ;
int	dirwid = 24 ;
int	bigwid = 15 ;

char	xdf_vertbar = '|' ;
char	xdf_horzbar = '-' ;
char	xdf_plusign = '+' ;
char	xdf_upprtee = 'v' ;
char	xdf_lowrtee = '^' ;
char	xdf_lefttee = '>' ;
char	xdf_rightee = '<' ;
char	xdf_upprleft = '/' ;
char	xdf_uppright = '\\' ;
char	xdf_lowrleft = '\\' ;
char	xdf_lowright = '/' ;

char *	xdf_smacs = "\033(0" ;
char *	xdf_rmacs = "\033(B" ;
char *	lotime = "localtime" ;
char *	osname = "Unix" ;
char *	osvers = "5" ;
char *	osrels = "4" ;
char *	forgid = SWFORG ;
char *	noname = "NodeName" ;
char *	hostname = "hostname" ;
char *	unitname = "bytes" ;

char	versno [16] ;
char	banner [80] ;
char	hident [128] ;

char * ignlst [] = { XDF_IGNLST , NULL } ;
char * syslst [] = { XDF_SYSLST , NULL } ;

char * dirlst [MAXDIRLST] ;

struct utsname	utsbuf ;

/*	__	__	__	__	__	__	__	__	*/

void wakeup (signo) {
	alarm (0) ;
	signal (SIGALRM, wakeup) ;
	alarm (0) ;
	++timedout ;
}

void dodraw (chr) int chr ; {
	if (ansiflag)
		printf ("%s%c%s", xdf_smacs, chr, xdf_rmacs) ;
	else
		printf ("%c", chr) ;
}

void showinfo (ptr, wid, flg) char * ptr ; int wid , flg ; {

	int		tmpint ;
	long long	tmpbig ;
	static	char	tmptxt [32] ;
	static	char	fmt [16] ;
		char *	tdalig = "right" ;

	if ( csvflag )
		wid = 0 ;

	if ( wid ) {
		if ( flg & XDF_STR ) {
			sprintf (fmt, "%%%d.%d", wid, wid < 0 ? -wid : wid) ;
		} else {
			sprintf (fmt, "%%%d", wid) ;
		}
	} else {
		sprintf (fmt, "%c", '%') ;
	}

	if ( wid < 0 )
		tdalig = "left" ;

	if ( flg & XDF_SOL ) {
		if ( htmlflag ) {
			if ( flg & XDF_HEAD ) {
				printf ("    <thead>\n") ;
				printf ("     <tr align=center>\n") ;
			}
			if ( flg & XDF_BODY ) {
				printf ("    <tbody>\n") ;
				printf ("     <tr align=left>\n") ;
			}
			if ( flg & XDF_FOOT ) {
				printf ("    <tfoot>\n") ;
				printf ("     <tr align=center>\n") ;
			}
		}
		if ( frameflag ) {
			dodraw (xdf_vertbar) ;
			printf ("%c", ' ') ;
		}
		return ;
	}

	if ( flg & XDF_UNT ) {
		if ( gigaflag && bigwid < 9 )
			unitname = "G" ;
		sprintf (tmptxt, "%s %s", ptr, unitname) ;
		ptr = tmptxt ;
	}

	if ( htmlflag ) {
		printf ("      <td align=%s>\t", tdalig) ;
	}

	if ( flg & XDF_STR ) {
		strcat (fmt, "s") ;
		printf (fmt, ptr) ;
	}

	if ( flg & XDF_INT ) {
		tmpint = *(int*)ptr ;
		strcat (fmt, "d") ;
		printf (fmt, tmpint) ;
	}

	if ( flg & XDF_BIG ) {
		tmpbig = *(long long *)ptr ;
		strcat (fmt, "lld") ;
		printf (fmt, tmpbig) ;
	}

	if ( flg & XDF_PCT ) {
		printf ("%c", '%') ;
	}

	if ( htmlflag ) {
		printf ("\t</td>\n") ;
	}

	if ( flg & XDF_EOL ) {
		if ( htmlflag ) {
			if ( flg & XDF_HEAD ) {
				printf ("     </tr>\n") ;
				printf ("    </thead>\n") ;
			}
			if ( flg & XDF_BODY ) {
				printf ("     </tr>\n") ;
				printf ("    </tbody>\n") ;
			}
			if ( flg & XDF_FOOT ) {
				printf ("     </tr>\n") ;
				printf ("    </tfoot>\n") ;
			}
		}
		if ( frameflag ) {
			printf ("%c", ' ') ;
			dodraw (xdf_vertbar) ;
		}
		if ( ! htmlflag )
			printf ("\n") ;
	} else {
		if ( csvflag ) {
			printf (",") ;
		} else {
			if ( frameflag ) {
				printf ("%c", ' ') ;
				dodraw (xdf_vertbar) ;
				printf ("%c", ' ') ;
			} else {
				if ( ! htmlflag )
					printf (" ") ;
			}
		}
	}
}

void xdf (vol, name) char * vol , * name ; {

	int	rd = -1 ;

	signal (SIGALRM, wakeup) ; timedout = 0 ; alarm (xtimeout) ;

# ifdef USE_STATFS
	rd = statfs (name, &sfsbuf) ;
# endif /* USE_STATFS */

# ifdef USE_STATVFS
	rd = statvfs (name, &svfsbuf) ;
# endif /* USE_STATVFS */

	alarm (0) ;

	if ( timedout ) {
		fprintf (stderr, "** timeout on %s\n", name != NULL ? name : vol != NULL ? vol : "unknown") ;
		return ;
	}

	if ( rd != 0 ) {
		perror (name) ;
		return ;
	}

# ifdef USE_STATFS
	x_bavail = sfsbuf.f_bavail ;
	x_bfree  = sfsbuf.f_bfree ;
	x_blocks = sfsbuf.f_blocks ;
	x_bsize  = sfsbuf.f_bsize ;
	xfs_blksiz = x_bsize ;
# endif /* USE_STATFS */

# ifdef USE_STATVFS
	x_bavail = svfsbuf.f_bavail ;
	x_bfree  = svfsbuf.f_bfree ;
	x_blocks = svfsbuf.f_blocks ;
	x_bsize  = svfsbuf.f_bsize ;
# ifdef HAS_VFS_F_SIZE
	x_size   = svfsbuf.f_size ;
# endif /* HAS_VFS_F_SIZE */
	x_frsize = svfsbuf.f_frsize ;
	xfs_blksiz = x_frsize ;
# endif /* USE_STATVFS */

	if ( x_blocks == 0 )
		return ;

	if ( vol == NULL )
		vol = "unknown" ;

	x_bused  = x_blocks - x_bfree ;

/* # include "xdfdbg.c" */

	xfs_totbyt = ( xfs_blksiz * x_blocks ) / unit ;
	xfs_usebyt = ( xfs_blksiz * x_bused ) / unit ;
	xfs_avabyt = ( xfs_blksiz * x_bavail ) / unit ;

	xfs_caperc = xfs_totbyt ? ( xfs_usebyt * 100 ) / xfs_totbyt : 0 ;
	xfs_tmperc = 100 - xfs_caperc ;

	showinfo (NULL, 0, XDF_SOL | XDF_ITEM) ;

	if ( hostflag )
		showinfo (hostname, -hostwid, XDF_STR) ;

	if ( volflag )
		showinfo (vol, -volwid, XDF_STR) ;

	if ( sizflag )
		showinfo ( (char *) &xfs_totbyt, bigwid, XDF_BIG) ;

	if ( useflag )
		showinfo ( (char *) &xfs_usebyt, bigwid, XDF_BIG) ;

	if ( avaflag )
		showinfo ( (char *) &xfs_avabyt, bigwid, XDF_BIG) ;

	if ( capflag )
		showinfo ( (char *) &xfs_caperc, 4, XDF_INT|XDF_PCT) ;

	if ( freeflag )
		showinfo ( (char *) &xfs_tmperc, 4, XDF_INT|XDF_PCT) ;

	showinfo (name, -dirwid, XDF_STR|XDF_EOL|XDF_ITEM) ;
	++volcount ;

	if ( sumflag ) {
		sumtotbyt += xfs_totbyt ;
		sumusebyt += xfs_usebyt ;
		sumavabyt += xfs_avabyt ;
		sumcaperc += xfs_caperc ;
		sumtmperc += xfs_tmperc ;
	}
}

void dodash (dash, plus, wid, flg) int dash, plus, wid, flg ; {
	int i ;

	for ( i = 2+wid ; i ; --i )
		dodraw (dash) ;
	if ( ! ( flg & XDF_EOL ) )
		dodraw (plus) ;
}

void dashes (flg) int flg ; {
	static	char	west, plus, east ;

	if ( ! frameflag )
		return ;

	if ( flg & XDF_HEAD ) {
		west = xdf_upprleft ;
		plus = xdf_upprtee ;
		east = xdf_uppright ;
	}

	if ( flg & XDF_PLUS ) {
		west = xdf_lefttee ;
		plus = xdf_plusign ;
		east = xdf_rightee ;
	}

	if ( flg & XDF_TAIL ) {
		west = xdf_lowrleft ;
		plus = xdf_lowrtee ;
		east = xdf_lowright ;
	}

	dodraw (west) ;

	if ( hostflag )
		dodash (xdf_horzbar, plus, hostwid, 0) ;

	if ( volflag )
		dodash (xdf_horzbar, plus, volwid, 0) ;

	if ( sizflag )
		dodash (xdf_horzbar, plus, bigwid, 0) ;

	if ( useflag )
		dodash (xdf_horzbar, plus, bigwid, 0) ;

	if ( avaflag )
		dodash (xdf_horzbar, plus, bigwid, 0) ;

	if ( capflag || freeflag )
		dodash (xdf_horzbar, plus, 5, 0) ;

	dodash (xdf_horzbar, plus, dirwid, XDF_EOL) ;
	dodraw (east) ;
	printf ("%c", '\n') ;
}

int chkign (name) char * name ; {

	char * * pp ;

	if ( name == NULL )
		return 1 ;

	for ( pp = ignlst ; *pp != NULL ; ++pp )
		if ( strcmp ( name , *pp ) == 0 )
			return 3 ;

	return 0 ;
}

int chksys (name) char * name ; {

	char * * pp ;

	if ( name == NULL )
		return 1 ;

	for ( pp = syslst ; *pp != NULL ; ++pp )
		if ( strcmp ( name , *pp ) == 0 )
			return 3 ;

	return 0 ;
}

void getuname () {
	int rd ;

	rd = uname (&utsbuf) ;

	if ( rd == -1 )
		perror ("uname") ;

	osname = utsbuf.sysname ;
	noname = utsbuf.nodename ;
	osvers = utsbuf.version ;
	osrels = utsbuf.release ;
}

void showhdr () {

	showinfo (NULL, 0, XDF_SOL | XDF_HEAD) ;
	if ( hostflag )
		showinfo ("host", -hostwid, XDF_STR /* | XDF_HEAD */) ;
	if ( volflag )
		showinfo ("volume", -volwid, XDF_STR /* | XDF_HEAD */) ;
	if ( sizflag )
		showinfo ("size", bigwid, XDF_STR | XDF_UNT /* | XDF_HEAD */) ;
	if ( useflag )
		showinfo ("used", bigwid, XDF_STR | XDF_UNT /* | XDF_HEAD */) ;
	if ( avaflag )
		showinfo ("free", bigwid, XDF_STR | XDF_UNT /* | XDF_HEAD */) ;
	if ( capflag )
		showinfo ("%used", 5, XDF_STR /* | XDF_HEAD */) ;
	if ( freeflag )
		showinfo ("%free", 5, XDF_STR /* | XDF_HEAD */) ;
	showinfo ("mounted", -dirwid, XDF_STR | XDF_EOL | XDF_HEAD) ;
	dashes (XDF_PLUS) ;
}

void showsums () {
	sumcaperc /= volcount ;
	sumtmperc /= volcount ;
	dashes (XDF_PLUS) ;
	showinfo (NULL, 0, XDF_SOL | XDF_FOOT) ;
	if ( hostflag )
		showinfo (hostname, -hostwid, XDF_STR) ;
	if ( volflag )
		showinfo ("total", -volwid, XDF_STR) ;
	if ( sizflag )
		showinfo ( (char *) &sumtotbyt, bigwid, XDF_BIG) ;
	if ( useflag )
		showinfo ( (char *) &sumusebyt, bigwid, XDF_BIG) ;
	if ( avaflag )
		showinfo ( (char *) &sumavabyt, bigwid, XDF_BIG) ;
	if ( capflag )
		showinfo ( (char *) &sumcaperc, 4, XDF_INT|XDF_PCT) ;
	if ( freeflag )
		showinfo ( (char *) &sumtmperc, 4, XDF_INT|XDF_PCT) ;
# ifdef ORIG
	showinfo ( capflag ? "average" : "total" , -dirwid, XDF_STR|XDF_EOL|XDF_FOOT ) ;
# else
	showinfo ( (char *) &volcount , -dirwid, XDF_INT|XDF_EOL|XDF_FOOT ) ;
# endif
}

int usage () {
	fprintf (stderr, " use: %s [-options]\n\n", SWNAME) ;
	fprintf (stderr, " -h : prepend host name\n" ) ;
	fprintf (stderr, " -v : show volume name\n" ) ;
	fprintf (stderr, " -s : show volume size\n" ) ;
	fprintf (stderr, " -u : show used space\n" ) ;
	fprintf (stderr, " -a : show available space\n" ) ;
	fprintf (stderr, " -c : show capacity ('%% full', or '%% used')\n" ) ;
	fprintf (stderr, " -f : show '%% free' instead (opposite of -c)\n" ) ;
	fprintf (stderr, " -K : show sizes in KiB\n" ) ;
	fprintf (stderr, " -M : show sizes in MiB\n" ) ;
	fprintf (stderr, " -G : show sizes in GiB\n" ) ;
	fprintf (stderr, " -O : show only 'operating system' (eg. /usr) filesystems\n" ) ;
	fprintf (stderr, " -R : show only 'regular' ('non-O.S.') filesystems\n" ) ;
	fprintf (stderr, " -N : show nfs filesystems\n" ) ;
	fprintf (stderr, " -A : frame output in ANSI line chars (*1)\n" ) ;
	fprintf (stderr, " -b : frame output in HTML format (*1)\n" ) ;
	fprintf (stderr, " -F : frame output in ASCII chars (*2)\n" ) ;
	fprintf (stderr, " -E : ignore 'echo' volumes (eg. cygwin)\n" ) ;
	fprintf (stderr, " -B : output informations in HTML format (*2)\n" ) ;
	fprintf (stderr, " -C : output informations in CSV format (*2)\n" ) ;
	fprintf (stderr, " -I : show host informations\n" ) ;
	fprintf (stderr, " -S : show total sizes and capacity average\n" ) ;
	fprintf (stderr, " -H : show header\n" ) ;
	fprintf (stderr, " -V : show version\n" ) ;
	fprintf (stderr, " -L : legacy df layout\n" ) ;
	fprintf (stderr, " -t : use 'static' (eg. fstab) filesystems list\n" ) ;
	fprintf (stderr, " -n <pathname> [ -n ... ] : process just these\n" ) ;
	fprintf (stderr, " -? : show usage help\n\n" ) ;
	fprintf (stderr, " (*1) '-A' and '-b' are mutually exclusive\n") ;
	fprintf (stderr, " (*2) '-B' , '-F' and '-C' are mutually exclusive\n") ;
	return 3 ;
}

void htmlhead () {
	printf ("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n") ;
	printf ("<html>\n") ;
	printf (" <head>\n") ;
	printf ("  <meta name=\"author\" content=\"Alexandre Botao, botao@unix.sh, botao@linux.sh, alexandre@botao.org\">\n") ;
	printf ("  <meta name=\"description\" content=\"%s\">\n", SWNAME) ;
	printf ("  <meta name=\"keywords\" content=\"%s,disk,free,usage,capacity\">\n", SWNAME) ;
	printf ("  <title> %s %s </title>\n", SWNAME, versno) ;
	printf ("  <style type=\"text/css\">\n") ;
	printf ("  p {color:cyan;font-family:Courier;}\n") ;
	printf ("  thead {color:blue;font-family:Courier;font-style:normal;font-weight:bold;}\n") ;
	printf ("  tbody {color:black;font-family:Courier;}\n") ;
	printf ("  tfoot {color:black;font-family:Courier;font-weight:bold;}\n") ;
	printf ("  td {padding:3px 5px;}\n") ;
	printf ("  caption {font-family:Arial;color:darkblue}\n") ;
	printf ("  </style>\n") ;
}

void htmlbody () {
	int bordersiz = 0 ;

	if ( borderflag )
		bordersiz = 4 ;
	printf (" </head>\n") ;
	printf (" <body>\n") ;
	printf ("   <p> %s </p>\n", banner) ;
	printf ("   <table border=s%d>\n", bordersiz) ;
	if ( identflag )
		printf ("    <caption> %s </caption>\n", hident) ;
}

void htmltail () {
	printf ("   </table>\n") ;
	printf (" </body>\n") ;
	printf ("</html>\n") ;
}

void showmesg (text) char * text ; {

	if ( htmlflag )
		return ;

	printf ("%s\n", text) ;
}

int ldf () {

	FILE * mfp ;
# ifdef SUNX
	struct mnttab mntbuf ;
# else
	struct mntent * mtp ;
# endif /* SUNX */
	char *	x_fsname; /* file system name */ 
	char *	x_dir;    /* file system path prefix */ 
	char *	x_type;
	char *	x_opts;
	char *	tp ;
	time_t	tloc ;
	int j ;

	getuname () ;

	strcpy (versno, SWVERS) ;

	if ( *forgid != '$' )
		strcat (versno, forgid) ;

	if ( ansiflag ) {
		xdf_vertbar = 'x' ;
		xdf_horzbar = 'q' ; /* ?r? */
		xdf_plusign = 'n' ;
		xdf_upprtee = 'w' ;
		xdf_lowrtee = 'v' ;
		xdf_lefttee = 't' ;
		xdf_rightee = 'u' ;
		xdf_upprleft = 'l' ;
		xdf_uppright = 'k' ;
		xdf_lowrleft = 'm' ;
		xdf_lowright = 'j' ;
		++frameflag ;
	}

	if ( hostflag ) {
		/* gethostname () ; */
		hostname = noname ;
	}

	if ( borderflag ) {
		++htmlflag ;
	}

	if ( frameflag ) {
		csvflag = htmlflag = 0 ;
	}

	if ( csvflag ) {
		frameflag = htmlflag = 0 ;
	}

	if ( htmlflag ) {
		csvflag = frameflag = 0 ;
	}

	strcpy (banner, SWNAME) ;

	if ( htmlflag )
		htmlhead () ;

	if ( verbflag ) {
		sprintf (banner, "%s %s %s %s %s", SWNAME, versno, SWDATE, SWCOPY, SWAUTH) ;
		showmesg (banner) ;
	}

	if ( identflag ) {
		time ( &tloc ) ;
		lotime = ctime ( &tloc ) ;
		tp = lotime + strlen (lotime) ;
		if ( *--tp == '\n' )
			*tp = '\0' ;
		sprintf (hident, "%s %s v%s r%s %s", noname, osname, osvers, osrels, lotime) ;
		showmesg (hident) ;
	}

	if ( helpflag )
		return usage () ;

	if ( htmlflag )
		htmlbody () ;

	if ( legflag ) {
		++volflag ;
		++sizflag ;
		++useflag ;
		++avaflag ;
		++capflag ;
		++hdrflag ;
		++kiloflag ;
	}

	if ( capflag ) {
		freeflag = 0 ; /* capflag prevails if both are inadvertently set */
	} else {
		if ( freeflag ) {
			capflag = 0 ; /* there can be only one ! */
		}
	}

	if ( kiloflag ) {
		unit = 1024LL ;
		bigwid = 12 ;
		unitname = "KiB" ;
	}

	if ( megaflag ) {
		unit = 1048576LL ;
		bigwid = 9 ;
		unitname = "MiB" ;
	}

	if ( gigaflag ) {
		unit = 1073741824LL ;
		bigwid = 6 ;
		unitname = "GiB" ;
	}

	dashes (XDF_HEAD) ;

	if ( hdrflag )
		showhdr () ;

	if ( listflag ) {
		for ( j = 0 ; j < listflag ; ++j ) {
			xdf (NULL, dirlst[j]) ;
		}
	} else {
		if ( fstabflag ) {
			mfp = setmntent ( X_FILESYSLIST , "r" ) ;
		} else {
			mfp = setmntent ( X_MOUNTEDLIST , "r" ) ;
		}
		if ( mfp == NULL ) {
			perror ( X_MOUNTEDLIST ) ;
			return 2 ;
		}
# ifdef SUNX
		while ( getmntent ( mfp , &mntbuf ) == 0 ) {
			x_fsname = mntbuf.mnt_special ;
			x_dir    = mntbuf.mnt_mountp ;
			x_type   = mntbuf.mnt_fstype ;
			x_opts   = mntbuf.mnt_mntopts ;
# else
		while ( ( mtp = getmntent ( mfp ) ) != NULL ) {
			x_fsname = mtp->mnt_fsname ;
			x_dir    = mtp->mnt_dir ;
			x_type   = mtp->mnt_type ;
			x_opts   = mtp->mnt_opts ;
# endif /* SUNX */
			if ( chkign ( x_type ) ) {
				continue ;
			}
			if ( 0 == strcmp ( x_type , "nfs" ) ) {
				if ( ! nfsflag ) {
					continue ;
				}
			}
			if ( sysflag ) {
				if ( ! chksys ( x_dir ) ) {
					continue ;
				}
			}
			if ( regflag ) {
				if ( chksys ( x_dir ) ) {
					continue ;
				}
			}
			if ( echoflag ) {
				if ( chkign ( x_fsname ) ) {
					continue ;
				}
			}
			xdf (x_fsname, x_dir) ;
		}
		endmntent ( mfp ) ;
	}

	if ( sumflag )
		showsums () ;

	dashes (XDF_TAIL) ;

	if ( htmlflag )
		htmltail () ;

	return 0 ;
}

int sdf (name) char * name ; {
	if ( name == NULL )
		return -1 ;
	if ( listflag == MAXDIRLST )
		return -1 ;
	dirlst[listflag++] = strdup (name) ;
	return 0 ;
}

int main (argc, argv) int argc ; char * * argv ; {

	if (--argc)
		while (*++argv)
			if ( strcmp ( *argv , "-K" ) == 0 )
				++kiloflag ;
			else if ( strcmp ( *argv , "-M" ) == 0 )
				++megaflag ;
			else if ( strcmp ( *argv , "-G" ) == 0 )
				++gigaflag ;
			else if ( strcmp ( *argv , "-T" ) == 0 )
				++teraflag ;
			else if ( strcmp ( *argv , "-F" ) == 0 )
				++frameflag ;
			else if ( strcmp ( *argv , "-I" ) == 0 )
				++identflag ;
			else if ( strcmp ( *argv , "-S" ) == 0 )
				++sumflag ;
			else if ( strcmp ( *argv , "-H" ) == 0 )
				++hdrflag ;
			else if ( strcmp ( *argv , "-V" ) == 0 )
				++verbflag ;
			else if ( strcmp ( *argv , "-A" ) == 0 )
				++ansiflag ;
			else if ( strcmp ( *argv , "-C" ) == 0 )
				++csvflag ;
			else if ( strcmp ( *argv , "-v" ) == 0 )
				++volflag ;
			else if ( strcmp ( *argv , "-s" ) == 0 )
				++sizflag ;
			else if ( strcmp ( *argv , "-u" ) == 0 )
				++useflag ;
			else if ( strcmp ( *argv , "-a" ) == 0 )
				++avaflag ;
			else if ( strcmp ( *argv , "-c" ) == 0 )
				++capflag ;
			else if ( strcmp ( *argv , "-L" ) == 0 )
				++legflag ;
			else if ( strcmp ( *argv , "-b" ) == 0 )
				++borderflag ;
			else if ( strcmp ( *argv , "-B" ) == 0 )
				++htmlflag ;
			else if ( strcmp ( *argv , "-E" ) == 0 )
				++echoflag ;
			else if ( strcmp ( *argv , "-O" ) == 0 )
				++sysflag ;
			else if ( strcmp ( *argv , "-R" ) == 0 )
				++regflag ;
			else if ( strcmp ( *argv , "-N" ) == 0 )
				++nfsflag ;
			else if ( strcmp ( *argv , "-h" ) == 0 )
				++hostflag ;
			else if ( strcmp ( *argv , "-f" ) == 0 )
				++freeflag ;
			else if ( strcmp ( *argv , "-t" ) == 0 )
				++fstabflag ;
			else if ( strcmp ( *argv , "-?" ) == 0 )
				++helpflag ;
			else if ( strcmp ( *argv , "-n" ) == 0 )
				sdf (*++argv) ;
			else
				fprintf (stderr, "%s: bad arg (%s)\n", SWNAME, *argv) ;

	return ldf () ;
}

/*
 * vi:nu ts=8
 */
